<?php

declare(strict_types=1);

namespace Drupal\cision;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\State\State;
use Drupal\key\KeyRepository;
use Psr\Http\Client\ClientInterface;

/**
 * Provides a class for interacting with the Cision API.
 */
final class Api {

  // Base URL for the Cision API.
  const BASE_URL = 'https://api.trendkite.com/api';

  // API Endpoints.
  const LOGIN_API = '/login';
  const TOTALMENTIONS = '/v2.2/totalmentions';
  const SEARCH_API = '/v2.2/searches';
  const STATS_API = '/v2.2/stats';

  // Header for the auth token.
  const AUTH_HEADER = 'X-Auth-Token';

  // Date format for the API.
  const DATE_FORMAT = 'Y-m-d\TG:i:s.vp';

  // Cache expiration times.
  const CACHE_EXPIRATION = 3600;

  // Auth token expiration time.
  const AUTH_TOKEN_EXPIRATION = 240;

  /**
   * Constructs an Api object.
   */
  public function __construct(
    private readonly ClientInterface $httpClient,
    private State $state,
    private readonly KeyRepository $keyRepository,
    private readonly DateFormatter $dateFormat,
    private readonly CacheBackendInterface $cache,
  ) {}

  /**
   * Get the auth token.
   *
   * @return string
   *   The auth token.
   */
  private function getAuthToken() {
    $token = $this->state->get('cision.auth_token');
    if ($token && $token['expires'] > time()) {
      return $token['token'];
    }

    $username = $this->keyRepository->getKey('cision_username')->getKeyValue();
    $password = $this->keyRepository->getKey('cision_password')->getKeyValue();
    if (!$username || !$password) {
      return NULL;
    }
    $data = $this->login($username, $password);
    if ($data) {
      $token = $data['access_token'];
      if ($token) {
        $this->state->set('cision.auth_token', [
          'token' => $token,
          'expires' => time() + self::AUTH_TOKEN_EXPIRATION,
        ]);
        return $token;
      }
    }
    return NULL;
  }

  /**
   * Login to the Cision API.
   *
   * @param string $username
   *   The username.
   * @param string $password
   *   The password.
   *
   * @return mixed
   *   The result of the login or null.
   */
  private function login($username, $password) {
    $uri = self::BASE_URL . self::LOGIN_API;
    $response = $this->httpClient->post($uri, [
      'json' => [
        'username' => $username,
        'password' => $password,
      ],
      'http_errors' => FALSE,
    ]);
    if ($response->getStatusCode() !== 200) {
      return NULL;
    }
    return json_decode($response->getBody()->getContents(), TRUE);
  }

  /**
   * Get Mentions for a search.
   *
   * @param int $s
   *   The search id.
   * @param int $range_start
   *   A Unix Timestamp with start of the date range.
   * @param int $range_end
   *   A Unix Timestamp with end of the date range.
   * @param int $page_num
   *   The page number.
   * @param int $page_size
   *   The page size.
   * @param string $sort
   *   The sort order, either asc or desc.
   * @param string $gaid
   *   The Google Analytics ID.
   *
   * @return mixed
   *   Get a list of totalmentions or null.
   */
  public function getTotalMentions($s, int $range_start, int $range_end, int $page_num = 0, int $page_size = 100, $sort = 'desc', $gaid = NULL) {
    $auth_token = $this->getAuthToken();
    if (!$auth_token) {
      return NULL;
    }

    $uri = self::BASE_URL . self::TOTALMENTIONS;

    $sort = $sort === 'desc' ? 'desc' : 'asc';

    $query = [
      's' => $s,
      'range-start' => $this->dateFormat->format($range_start, 'custom', self::DATE_FORMAT),
      'range-end' => $this->dateFormat->format($range_end, 'custom', self::DATE_FORMAT),
      'page-num' => $page_num,
      'page-size' => $page_size,
      'sort' => $sort,
      'format' => 'json',
    ];

    if ($gaid) {
      $query['gaid'] = $gaid;
    }

    $cid = 'cision:totalmentions:' . md5(serialize($query));
    if ($cache = $this->cache->get($cid)) {
      return $cache->data;
    }

    $response = $this->httpClient->get($uri, [
      'headers' => [self::AUTH_HEADER => $auth_token],
      'query' => $query,
      'http_errors' => FALSE,
    ]);

    if ($response->getStatusCode() !== 200) {
      return NULL;
    }

    $data = json_decode($response->getBody()->getContents(), TRUE);
    $this->cache->set($cid, $data, time() + self::CACHE_EXPIRATION);
    return $data;
  }

  /**
   * Get Searches.
   *
   * @param bool $shared
   *   Whether to include shared searches.
   *
   * @return mixed
   *   The list of searches or null.
   */
  public function getSearches($shared = TRUE) {
    $cid = 'cision:searches:' . $shared ? 'true' : 'false';
    if ($cache = $this->cache->get($cid)) {
      return $cache->data;
    }
    $auth_token = $this->getAuthToken();
    if (!$auth_token) {
      return NULL;
    }

    $uri = self::BASE_URL . self::SEARCH_API;
    $response = $this->httpClient->get($uri, [
      'headers' => [
        self::AUTH_HEADER => $auth_token,
      ],
      'query' => [
        'shared' => $shared ? 'true' : 'false',
      ],
      'http_errors' => FALSE,
    ]);
    if ($response->getStatusCode() !== 200) {
      return NULL;
    }

    $data = json_decode($response->getBody()->getContents(), TRUE);
    $this->cache->set($cid, $data['searches'], time() + self::CACHE_EXPIRATION);
    return $data['searches'];
  }

  /**
   * Get Searches Options.
   *
   * @return array
   *   The list of searches options.
   */
  public function getSearchesOptions(): array {
    $cid = 'cision:searchesOptions';
    if ($cache = $this->cache->get($cid)) {
      return $cache->data;
    }
    $searches = $this->getSearches();
    $options = [];
    foreach ($searches as $search) {
      $options[$search['id']] = $search['title'];
    }
    $this->cache->set($cid, $options, time() + self::CACHE_EXPIRATION);
    return $options;
  }

  /**
   * Get Search stats.
   *
   * @param int $s
   *   The search id.
   * @param int $range_start
   *   A Unix Timestamp with start of the date range.
   * @param int $range_end
   *   A Unix Timestamp with end of the date range.
   * @param string $type
   *   The type of stats to get.
   *   Either: 'all', 'ave', 'readership', 'totalMentions', or 'socialShares'.
   *
   * @return mixed
   *   The stats or null.
   */
  public function getStats($s, int $range_start, int $range_end, $type = 'all') {
    switch ($type) {
      case 'ave':
        $type = 'ave';
        break;

      case 'readership':
        $type = 'readership';
        break;

      case 'totalMentions':
        $type = 'totalMentions';
        break;

      case 'socialShares':
        $type = 'socialShares';
        break;

      default:
        $type = 'all';
    }

    $auth_token = $this->getAuthToken();
    if (!$auth_token) {
      return NULL;
    }

    $uri = self::BASE_URL . self::STATS_API;

    $query = [
      's' => $s,
      'range-start' => $this->dateFormat->format($range_start, 'custom', self::DATE_FORMAT),
      'range-end' => $this->dateFormat->format($range_end, 'custom', self::DATE_FORMAT),
      'type' => $type,
    ];

    $cid = 'cision:stats:' . md5(serialize($query));
    if ($cache = $this->cache->get($cid)) {
      return $cache->data;
    }

    $response = $this->httpClient->get($uri, [
      'headers' => [self::AUTH_HEADER => $auth_token],
      'query' => $query,
      'http_errors' => FALSE,
    ]);

    if ($response->getStatusCode() !== 200) {
      return NULL;
    }

    $data = json_decode($response->getBody()->getContents(), TRUE);
    $this->cache->set($cid, $data, time() + self::CACHE_EXPIRATION);
    return $data;
  }

}
