<?php

declare(strict_types=1);

namespace Drupal\cision\Plugin\Block;

use Drupal\Core\Block\Attribute\Block;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\cision\Api;
use Embed\Embed;
use Embed\Http\Crawler;
use Embed\Http\CurlClient;
use Exception;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a cision total mentions block.
 */
#[Block(
  id: 'cision_cision_total_mentions',
  admin_label: new TranslatableMarkup('Cision Total Mentions'),
  category: new TranslatableMarkup('Cision'),
)]
final class CisionTotalMentionsBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Constructs the plugin instance.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    private readonly Api $cisionApi,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new self(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('cision.api'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    $placeholder_image_id = \Drupal::configFactory()->get('cision.settings')->get('placeholder_image_id');
    $placeholder_file = $placeholder_image_id ? \Drupal\media\Entity\Media::load($placeholder_image_id)->field_media_image : NULL;
    if (is_object($placeholder_file)) {
      $placeholder_id = $placeholder_file->target_id;
    }

    return [
      'sid' => NULL,
      'start_date' => 'last year',
      'end_date' => 'now',
      'max_results' => 8,
      'remove_duplicates' => FALSE,
      'image_style' => NULL,
      'placeholder_image' => $placeholder_id ? $placeholder_id : NULL,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state): array {
    // Load all image styles.
    $image_styles = \Drupal\image\Entity\ImageStyle::loadMultiple();
    $options = [];

    foreach ($image_styles as $style) {
      $options[$style->id()] = $style->label();
    }

    $form['sid'] = [
      '#type' => 'select',
      '#title' => $this->t('Search'),
      '#default_value' => $this->configuration['sid'],
      '#options' => $this->cisionApi->getSearchesOptions(),
    ];

    $form['start_date'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Start Date'),
      '#default_value' => $this->configuration['start_date'],
      '#description' => $this->t('This date can be entered in any formet supported by strtotime (https://www.php.net/manual/en/function.strtotime.php).'),
    ];

    $form['end_date'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Search ID'),
      '#default_value' => $this->configuration['end_date'],
      '#description' => $this->t('This date can be entered in any formet supported by strtotime (https://www.php.net/manual/en/function.strtotime.php).'),
    ];

    $form['max_results'] = [
      '#type' => 'number',
      '#title' => $this->t('Max Results'),
      '#default_value' => $this->configuration['max_results'],
    ];

    $form['remove_duplicates'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Remove Duplicates'),
      '#description' => $this->t('This will remove duplicated articles, if duplicates are present the number of results will be less.'),
      '#default_value' => $this->configuration['remove_duplicates'],
    ];

    $form['image_style'] = [
      '#type' => 'select',
      '#title' => $this->t('Image Style'),
      '#default_value' => $this->configuration['image_style'],
      '#options' => $options,
    ];

    $form['placeholder_image'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Placeholder Image'),
      '#default_value' => [$this->configuration['placeholder_image']],
      '#upload_location' => 'public://cision_placeholder_images',
      '#description' => $this->t('A placeholder image for cases where there is no image provided.'),
      '#multiple' => FALSE,
      '#upload_validators' => [
        'file_validate_extensions' => ['png jpg jpeg'],
      ],
      '#required' => TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockValidate($form, FormStateInterface $form_state) {
    if (!strtotime($form_state->getValue('start_date'))) {
      $form_state->setErrorByName('start_date', $this->t('Invalid start date.'));
    }

    if (!strtotime($form_state->getValue('end_date'))) {
      $form_state->setErrorByName('end_date', $this->t('Invalid end date.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state): void {
    // Save image as permanent.
    $image = $form_state->getValue('placeholder_image');
    if ($image != $this->configuration['placeholder_image']) {
      if (!empty($image[0])) {
        $file = \Drupal\file\Entity\File::load($image[0]);
        $file->setPermanent();
        $file->save();
      }
    }

    $this->configuration['sid'] = $form_state->getValue('sid');
    $this->configuration['start_date'] = $form_state->getValue('start_date');
    $this->configuration['end_date'] = $form_state->getValue('end_date');
    $this->configuration['max_results'] = (int) $form_state->getValue('max_results');
    $this->configuration['remove_duplicates'] = $form_state->getValue('remove_duplicates');
    $this->configuration['image_style'] = $form_state->getValue('image_style');
    $this->configuration['placeholder_image'] = implode($form_state->getValue('placeholder_image'));

  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    $total_mentions = $this->cisionApi->getTotalMentions(
      $this->configuration['sid'],
      strtotime($this->configuration['start_date']),
      strtotime($this->configuration['end_date']),
      0,
      $this->configuration['max_results']
    );

    // If there are no mentions, skip the rest.
    if (!is_array($total_mentions)) {
      return [];
    }

    foreach ($total_mentions as $key => $mention) {
      if (filter_var($mention['url'], FILTER_VALIDATE_URL) === FALSE) {
        unset($total_mentions[$key]);
      }
    }
    $total_mentions = array_values($total_mentions);

    if (!$total_mentions) {
      return [
        '#cache' => [
          'max-age' => 0,
        ],
      ];
    }

    // Check for duplicate articles from separate sites.
    if ($this->configuration['remove_duplicates']) {
      $titles = [];

      foreach ($total_mentions as $key => $mention) {
        if (!in_array(strtolower($mention['title']), $titles)) {
          $titles[] = strtolower($mention['title']);
        }
        else {
          unset($total_mentions[$key]);
        }
      }
    }

    $urls = [];
    foreach ($total_mentions as $key => $mention) {
      if (!str_starts_with($mention['url'], 'https')) {
        unset($total_mentions[$key]);
      }
      else {
        $urls[] = $mention['url'];
      }
    }

    $total_mentions = array_values($total_mentions);

    $client = new CurlClient();
    $client->setSettings([
      'timeout' => 10,
    ]);

    $embed = new Embed(new Crawler($client));

    try {
      $infos = $embed->getMulti(...$urls);
    }
    catch (Exception $e) {
      $total_mentions = [];
    }

    for ($x = 0; $x < count($total_mentions); $x++) {
      $total_mentions[$x]['image'] = $infos[$x];
    }

    $build = [];

    $placeholder_image_url = \Drupal\file\Entity\File::load($this->configuration['placeholder_image'])->uri;

    $build['content'] = [
      '#theme' => 'cision_item_list',
      '#image_style' => $this->configuration['image_style'],
      '#placeholder_image' => $placeholder_image_url ? $placeholder_image_url : NULL,
      '#items' => array_map(function ($mention) {
        $image = $mention['image']->image;
        $image_url = $image ? 'http://' . $image->getHost() . $image->getPath() : '';

        if (!str_ends_with($image_url, "jpg") && !str_ends_with($image_url, "jpeg") && !str_ends_with($image_url, "png")) {
          $image_url = NULL;
        }

        return [
          'mediaType' => $mention['mediaType'],
          'title' => $mention['title'] ,
          'date' => $mention['date'],
          'url' => $mention['url'],
          'imageURL' => $image_url,
        ];

      }, $total_mentions),
      '#attributes' => ['class' => ['cision-list']],
      '#cache' => [
        'max-age' => $this->cisionApi::CACHE_EXPIRATION,
      ],
      '#attached' => [
        'library' => [
          'cision/cision',
        ],
      ],
    ];
    return $build;
  }

}
